import { __awaiter } from "tslib";
import { DesignSystem, DI } from '@microsoft/fast-foundation';
import { fixture as FASTFixture } from '../component';
import { assert, suite } from '../uvu';
/**
 * Create component test suite.
 *
 * @example
 * Simple suite using the tag name of the component.
 * ```ts
 * import { createComponentSuite } from '@genesislcap/foundation-testing';
 * import { Component } from './component';
 * Component; // < As we're using tag name in the Suite, we hold a reference to avoid tree shaking.
 * const Suite = createComponentSuite<Component>('Component', 'my-component');
 * // test cases...
 * Suite.run();
 * ```
 *
 * @example
 * Mocking a DI dependency for a composable component.
 * ```ts
 * const connectMock = new ConnectMock();
 * const mocks = [Registration.instance(Connect, connectMock)];
 * const Suite = createComponentSuite<ConnectionIndicator>('ConnectionIndicator Component', () => connectionIndicator(), null, mocks);
 * // test cases...
 * Suite.run();
 * ```
 *
 * @example
 * An element will be required to test anything that directly or in-directly makes use of the DI container, for example,
 * a service that can be injected into components, or has its own injected dependencies.
 * ```ts
 * import { Service } from './service';
 * @customElement({
 *   name: 'test-element',
 *   template: html`<slot></slot>`,
 * })
 * class TestElement extends FASTElement {}
 * const mocks = [...];
 * const Suite = createComponentSuite<TestElement>('Service', 'test-element', null, mocks);
 * // test cases...
 * Suite.run();
 * ```
 *
 * Importing the service should invoke the Service's DI registration, so in your test cases you can simply query the
 * container to get a reference to your service.
 * ```ts
 * Suite('Service.x does something expected', async ({ container }) => {
 *   const myService = container.get(Service);
 *   // assert
 * });
 * ```
 *
 * You can optionally add the service to the test element for lookup convenience, but this is not required.
 * ```ts
 * class TestElement extends FASTElement {
 *   @Service service: Service;
 * }
 * Suite('Element has service injected', async ({ element }) => {
 *   assert.ok(element.service);
 * });
 * ```
 *
 * @typeParam TElement - The element interface.
 * @param title - Title of the test suite
 * @param elementNameOrGetter - Element tag name or getter which is used to create the element within the fixture
 * @param context - Optional component context {@link ComponentContext}
 * @param registrations - Optional array of DI container registrations
 * @returns The test suite
 * @public
 * @remarks
 * Used to test function output given certain input arguments.
 */
export function createComponentSuite(title, elementNameOrGetter, context, registrations = []) {
    const Suite = suite(title, Object.assign({ element: undefined, disconnect: undefined, designSystem: undefined, container: undefined }, context));
    Suite.before.each((c) => __awaiter(this, void 0, void 0, function* () {
        const parent = document.createElement('div');
        const designSystem = DesignSystem.getOrCreate(parent).register(...registrations);
        const container = DI.getOrCreateDOMContainer(parent);
        const fixtureOptions = {
            parent,
            designSystem,
        };
        /**
         * We might need to spread element getters for some test case, for example look at the following Fast test:
         * packages/web-components/fast-foundation/src/picker/picker.spec.ts
         */
        const { element, connect, disconnect } = yield FASTFixture(typeof elementNameOrGetter === 'string' ? elementNameOrGetter : elementNameOrGetter(), fixtureOptions);
        c.element = element;
        c.disconnect = disconnect;
        c.designSystem = designSystem;
        c.container = container;
        yield connect();
    }));
    /* eslint-disable require-atomic-updates */
    Suite.after.each((c) => __awaiter(this, void 0, void 0, function* () {
        yield c.disconnect();
        c.element = undefined;
        c.disconnect = undefined;
        c.designSystem = undefined;
        c.container = undefined;
    }));
    return Suite;
}
/**
 * Create logic test suite.
 *
 * @example
 * ```ts
 * import { createLogicSuite } from '@genesislcap/foundation-testing';
 * import { myFunction } from './logic';
 * const Suite = createLogicSuite('myFunction');
 * Suite('myFunction should provide expected results', ({ runCases }) => {
 *   runCases(myFunction, [
 *     [['1'], true],
 *     [[123], true],
 *     [['60%'], true],
 *     [['$60'], false],
 *     [['1.1'], false],
 *     [[''], false],
 *     [[true], false],
 *     [[null], false],
 *     [[undefined], false],
 *   ]);
 * });
 * Suite.run();
 * ```
 *
 * @typeParam TContext - The context interface.
 * @param title - Title of the test suite
 * @param context - Optional context which extends {@link LogicContext}
 * @returns The test suite
 * @public
 * @remarks
 * Used to test function output given certain input arguments.
 */
export function createLogicSuite(title, context) {
    const Suite = suite(title, Object.assign({ runCases: (fn, cases) => {
            assert.type(fn, 'function');
            cases.forEach(([actual, expects, msg]) => assert.is(fn(...actual), expects, msg));
        } }, context));
    return Suite;
}
